import { Autoload, Init, Scope, ScopeEnum } from '@midwayjs/core';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { Types } from 'mongoose';
import { Admin } from '../../entity/system/admin.entity';
import { DictData } from '../../entity/system/dictData.entity';
import { DictType } from '../../entity/system/dictType.entity';
import { Menus } from '../../entity/system/menus.entity';
import { Role } from '../../entity/system/role.entity';
const ObjectId = Types.ObjectId;
@Autoload()
@Scope(ScopeEnum.Singleton)
export class InitService {
  @InjectEntityModel(Admin)
  adminModel: ReturnModelType<typeof Admin>;
  @InjectEntityModel(Role)
  roleModel: ReturnModelType<typeof Role>;
  @InjectEntityModel(Menus)
  menusModel: ReturnModelType<typeof Menus>;
  @InjectEntityModel(DictType)
  dictTypeModel: ReturnModelType<typeof DictType>;
  @InjectEntityModel(DictData)
  dictDataModel: ReturnModelType<typeof DictData>;
  @Init()
  async init() {
    // 都检查是否有数据,没有数据才初始化
    this.adminUser();
    this.initRole();
    this.initMenus();
    this.initDict();
  }

  async adminUser() {
    const data = {
      account: 'sadmin',
      password: '1qaz2wsx',
      nick_name: '系统管理员',
      is_super: '0',
    };
    const is_exist = await this.adminModel.count({ is_super: '0' });
    if (!is_exist) return await this.adminModel.create(data);
    return;
  }

  async initRole() {
    const num = await this.roleModel.count();
    if (num > 0) return;
    const datas = [{ name: '管理员', code: 'Admin' }];
    await this.roleModel.insertMany(datas);
  }
  async initMenus() {
    const num = await this.menusModel.count();
    if (num > 0) return;
    const datas: any = [
      {
        name: '首页',
        icon: 'House',
        order_num: 1,
        path: '/',
        component: '/home/index',
        type: '1',
        route_name: 'home',
      },
    ];
    // 系统设置
    const smId = new ObjectId();
    const systemMenus = [
      {
        _id: smId,
        name: '系统设置',
        icon: 'Setting',
        path: '/system',
        order_num: 2,
        type: '0',
        route_name: 'system',
        is_default: '0',
      },
      {
        name: '菜单设置',
        icon: 'Setting',
        parent_id: smId.toString(),
        order_num: 1,
        path: '/system/menus',
        component: '/system/menus/index',
        type: '1',
        route_name: 'system_menus',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '添加下一级',
            code: 'addNext',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
        is_default: '0',
      },
      {
        name: '角色管理',
        icon: 'Setting',
        parent_id: smId.toString(),
        order_num: 2,
        path: '/system/role',
        component: '/system/role/index',
        type: '1',
        route_name: 'system_role',
        is_default: '0',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '使用',
            code: 'toAbled',
          },
          {
            zh: '禁用',
            code: 'toDisabled',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '字典管理',
        icon: 'Setting',
        parent_id: smId.toString(),
        order_num: 3,
        path: '/system/dict',
        component: '/system/dict/index',
        type: '1',
        route_name: 'system_dict',
        is_default: '0',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '字典数据',
            code: 'dict',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '字典数据',
        icon: 'Setting',
        parent_id: smId.toString(),
        order_num: 4,
        path: '/system/dictData',
        component: '/system/dictData/index',
        type: '2',
        route_name: 'system_dict_data',
        is_default: '0',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '组织管理',
        icon: 'Setting',
        parent_id: smId.toString(),
        order_num: 3,
        path: '/system/organization',
        component: '/system/organization/index',
        type: '1',
        route_name: 'system_organization',
        is_default: '0',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
    ];
    // 用户管理
    const umId = new ObjectId();
    const userMenus = [
      {
        _id: umId,
        name: '用户管理',
        icon: 'User',
        order_num: 3,
        path: '/user',
        type: '0',
        route_name: 'user',
        is_default: '1',
      },
      {
        name: '管理员用户',
        icon: 'User',
        parent_id: umId.toString(),
        order_num: 1,
        path: '/user/admin',
        component: '/user/admin/index',
        type: '1',
        route_name: 'user_admin',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '重置密码',
            code: 'rp',
          },
          {
            zh: '使用/禁用',
            code: 'changeUse',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
    ];
    // 作业管理
    const jmId = new ObjectId();
    const jobMenus = [
      {
        _id: jmId,
        name: '作业管理',
        icon: 'TakeawayBox',
        order_num: 4,
        path: '/job',
        type: '0',
        route_name: 'job',
        is_default: '1',
      },
      {
        name: '实时作业',
        icon: 'TakeawayBox',
        parent_id: jmId.toString(),
        order_num: 1,
        path: '/job/realtime',
        component: '/job/realtime/index',
        type: '1',
        route_name: 'job_realtime',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '历史作业',
        icon: 'TakeawayBox',
        parent_id: jmId.toString(),
        order_num: 2,
        path: '/job/history',
        component: '/job/history/index',
        type: '1',
        route_name: 'job_history',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '监控作业',
        icon: 'TakeawayBox',
        parent_id: jmId.toString(),
        order_num: 3,
        path: '/job/monitor',
        component: '/job/monitor/index',
        type: '1',
        route_name: 'job_monitor',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
    ];
    // 集群管理
    const colonymId = new ObjectId();
    const colonyMenus = [
      {
        _id: colonymId,
        name: '集群管理',
        icon: 'Histogram',
        order_num: 5,
        path: '/colony',
        type: '0',
        route_name: 'colony',
        is_default: '1',
      },
      {
        name: '分区管理',
        icon: 'Histogram',
        parent_id: colonymId.toString(),
        order_num: 1,
        path: '/colony/partition',
        component: '/colony/partition/index',
        type: '1',
        route_name: 'colony_partition',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '节点管理',
        icon: 'Histogram',
        parent_id: colonymId.toString(),
        order_num: 2,
        path: '/colony/node',
        component: '/colony/node/index',
        type: '1',
        route_name: 'colony_node',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
    ];
    // 监控管理
    const controlmId = new ObjectId();
    const controlMenus = [
      {
        _id: controlmId,
        icon: 'DataLine',
        name: '监控管理',
        order_num: 6,
        path: '/control',
        type: '0',
        route_name: 'control',
        is_default: '1',
      },
      {
        name: '资源管理',
        icon: 'DataLine',
        parent_id: controlmId.toString(),
        order_num: 1,
        path: '/control/resource',
        component: '/control/resource/index',
        type: '1',
        route_name: 'control_resource',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '性能管理',
        icon: 'DataLine',
        parent_id: controlmId.toString(),
        order_num: 2,
        path: '/control/performance',
        component: '/control/performance/index',
        type: '1',
        route_name: 'control_performance',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '报表管理',
        icon: 'DataLine',
        parent_id: controlmId.toString(),
        order_num: 3,
        path: '/control/statement',
        component: '/control/statement/index',
        type: '1',
        route_name: 'control_statement',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'edit',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
    ];
    const operate = {
      name: '操作日志',
      icon: 'Compass',
      order_num: 7,
      path: '/operate',
      component: '/operate/index',
      type: '1',
      route_name: 'operate',
    };
    const password = {
      name: '个人中心',
      icon: 'Reading',
      order_num: 999,
      path: '/account',
      component: '/account/index',
      type: '1',
      route_name: 'account',
    };

    datas.push(...systemMenus, ...userMenus, password);
    // 项目业务菜单
    const busMenus = [...jobMenus, ...colonyMenus, ...controlMenus, operate];
    datas.push(...busMenus);
    await this.menusModel.insertMany(datas);
  }
  async initRoleMenu(admin: Admin) {}

  async initDict() {
    const num = await this.dictTypeModel.count();
    if (num > 0) return;
    const isUseType = [
      { title: '是否使用', code: 'isUse', is_use: '0' },
      { title: '图标', code: 'icon', is_use: '0' },
    ];
    const isUseData = [
      { code: 'isUse', label: '使用', value: '0', sort: 1, is_use: '0' },
      { code: 'isUse', label: '禁用', value: '1', sort: 2, is_use: '0' },
      { code: 'icon', label: 'House', value: 'House', sort: 1, is_use: '0' },
      {
        code: 'icon',
        label: 'Setting',
        value: 'Setting',
        sort: 2,
        is_use: '0',
      },
      { code: 'icon', label: 'User', value: 'User', sort: 3, is_use: '0' },
      {
        code: 'icon',
        label: 'Compass',
        value: 'Compass',
        sort: 4,
        is_use: '0',
      },
      {
        code: 'icon',
        label: 'Histogram',
        value: 'Histogram',
        sort: 5,
        is_use: '0',
      },
      {
        code: 'icon',
        label: 'DataLine',
        value: 'DataLine',
        sort: 6,
        is_use: '0',
      },
      {
        code: 'icon',
        label: 'TakeawayBox',
        value: 'TakeawayBox',
        sort: 7,
        is_use: '0',
      },
      {
        code: 'icon',
        label: 'Unlock',
        value: 'Unlock',
        sort: 8,
        is_use: '0',
      },
      {
        code: 'icon',
        label: 'Reading',
        value: 'Reading',
        sort: 9,
        is_use: '0',
      },
    ];
    await this.dictTypeModel.insertMany(isUseType);
    await this.dictDataModel.insertMany(isUseData);
  }
}
